#include "stdafx.h"
#include "CameraControl.h"
#include "World.h"
#include ".\cameracontrol.h"


IMPLEMENT_DYNAMIC(CameraControl, CDialog)
CameraControl::CameraControl(CWnd* pParent)
: CDialog(CameraControl::IDD, pParent)
, m_bMoveEye(FALSE)
, m_bShowCamera(FALSE)
, m_bCameraRail(FALSE)
, m_pMainViewCamera(NULL)
, m_pZoomViewCamera(NULL)
, m_CameraZoomType(eCameraZoomDistance)
{
}

CameraControl::~CameraControl()
{
}

void CameraControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SLIDER_CAMERA, m_CameraSlider);
  DDX_Check(pDX, IDC_CHECK_MOVE_EYE, m_bMoveEye);
  DDX_Check(pDX, IDC_CHECK_SHOW_CAMERA, m_bShowCamera);
  DDX_Check(pDX, IDC_CHECK_CAMERA_RAIL, m_bCameraRail);
}


BEGIN_MESSAGE_MAP(CameraControl, CDialog)
  ON_BN_CLICKED(IDC_CHECK_SHOW_CAMERA, OnBnClickedCheckShowCamera)
  ON_BN_CLICKED(IDC_CHECK_CAMERA_RAIL, OnBnClickedCheckCameraRail)
  ON_WM_HSCROLL()
  ON_BN_CLICKED(IDC_FOV_ZOOM, OnBnClickedFovZoom)
  ON_BN_CLICKED(IDC_DISTANCE_ZOOM, OnBnClickedDistanceZoom)
  ON_BN_CLICKED(IDC_CHECK_MOVE_EYE, OnBnClickedCheckMoveEye)
END_MESSAGE_MAP()


void CameraControl::OnBnClickedCheckShowCamera()
{
  UpdateData(TRUE);
  theWorld.m_bShowCamera = (m_bShowCamera == TRUE);
}

void CameraControl::OnBnClickedCheckCameraRail()
{
  UpdateData(TRUE);
  theWorld.m_bShowCameraRail = (m_bCameraRail == TRUE);
}

BOOL CameraControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  // call set camera to sync all the child controls to the current camera settings
  if(m_pMainViewCamera && m_pZoomViewCamera) SetCameras(m_pMainViewCamera, m_pZoomViewCamera);

  CheckRadioButton(IDC_FOV_ZOOM, IDC_DISTANCE_ZOOM, IDC_DISTANCE_ZOOM);
  m_bShowCamera = theWorld.m_bShowCamera;
  m_bCameraRail = theWorld.m_bShowCameraRail;
  UpdateData(FALSE);
  return TRUE;
}

void CameraControl::SetCameras(Camera* pMainViewCamera, Camera* pZoomViewCamera)
{
  m_pMainViewCamera = pMainViewCamera;
  m_pZoomViewCamera = pZoomViewCamera;
  if(NULL == m_pMainViewCamera || NULL == m_pZoomViewCamera) return;

  if(IsWindow(m_hWnd))
  {
    m_CameraSlider.Initialize(1.0f,30.0f,m_pMainViewCamera->GetViewDistance());
    m_bMoveEye = TRUE;
    UpdateData(FALSE);
  }
}

void CameraControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  SliderCtrlWithEcho *pSlider = &m_CameraSlider;
  if(eCameraZoomDistance == m_CameraZoomType)
  {
    m_pMainViewCamera->SetViewDistance(pSlider->GetSliderValue(), (m_bMoveEye == TRUE) );
  }
  else
  {
    m_pMainViewCamera->SetFieldOfView(D3DXToRadian(pSlider->GetSliderValue()));
  }
  pSlider->HScroll(nSBCode, nPos);
}
void CameraControl::OnBnClickedFovZoom()
{
  m_CameraZoomType = eCameraZoomFOV;
  m_CameraSlider.Initialize(1.0f,180.0f,m_pMainViewCamera->GetViewDistance());
}

void CameraControl::OnBnClickedDistanceZoom()
{
  m_CameraZoomType = eCameraZoomDistance;
  m_CameraSlider.Initialize(1.0f,30.0f,m_pMainViewCamera->GetFieldOfView()*180.0f/M_PI);
}

void CameraControl::OnBnClickedCheckMoveEye()
{
  UpdateData(TRUE);
}
