#include "stdafx.h"
#include "ClassExampleApp.h"
#include "ClassExampleDlg.h"
#include "World.h"
#include ".\classexampledlg.h"

//Always include the following at the top of your .cpp files in MFC projects. This
//makes it much easier to track down memory leaks
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif


ClassExampleDlg::ClassExampleDlg(CWnd* pParent)
: CDialog(ClassExampleDlg::IDD, pParent)
, m_ShowCameraControl(FALSE)
, m_ShowLightControl(FALSE)
, m_TriangleProblem(FALSE)
, m_PerspectiveProblem(FALSE)
, m_SilhouetteProblem(FALSE)
, m_NormalProblem(FALSE)
, m_TJunctionProblem(FALSE)
{
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void ClassExampleDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Check(pDX, IDC_SHOW_CAMERAL_CONTROL, m_ShowCameraControl);
  DDX_Check(pDX, IDC_SHOW_LIGHT_CONTROL, m_ShowLightControl);
  DDX_Check(pDX, IDC_TRIANGLE_ORIENTATION, m_TriangleProblem);
  DDX_Check(pDX, IDC_PERSPECTIVE_DISTORTION, m_PerspectiveProblem);
  DDX_Check(pDX, IDC_SILHOUETTE, m_SilhouetteProblem);
  DDX_Check(pDX, IDC_NORMAL_PROBLEM, m_NormalProblem);
  DDX_Check(pDX, IDC_T_JUNCTION, m_TJunctionProblem);
}

BEGIN_MESSAGE_MAP(ClassExampleDlg, CDialog)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  //}}AFX_MSG_MAP
  ON_WM_TIMER()
  ON_BN_CLICKED(IDC_RESET_WORLD, OnBnClickedResetWorld)
  ON_BN_CLICKED(IDC_SHOW_CAMERAL_CONTROL, OnBnClickedShowCameralControl)
  ON_BN_CLICKED(IDC_SHOW_LIGHT_CONTROL, OnBnClickedShowLightControl)
  ON_BN_CLICKED(IDC_TRIANGLE_ORIENTATION, OnBnClickedTriangleOrientation)
  ON_BN_CLICKED(IDC_PERSPECTIVE_DISTORTION, OnBnClickedPerspectiveDistortion)
  ON_BN_CLICKED(IDC_SILHOUETTE, OnBnClickedSilhouette)
  ON_BN_CLICKED(IDC_NORMAL_PROBLEM, OnBnClickedNormalProblem)
  ON_BN_CLICKED(IDC_T_JUNCTION, OnBnClickedTJunction)
END_MESSAGE_MAP()



BOOL ClassExampleDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);   // Set big icon
  SetIcon(m_hIcon, FALSE);  // Set small icon

  // Initialize our D3D pipeline connection. This is best done at application initialization
  if(!GraphicsSystem::Initialize()) return FALSE;

  //Create a D3D Window. We do this by replacing a control on our dialog. The dialog
  //control is simply used as a placeholder to define our window's initial size and position.
  if(!m_MainUIWindow.ReplaceDialogControl(m_hWnd,IDC_MAIN_VIEW_PLACEHOLDER)) return FALSE;
  if(!m_CornerUIWindow.ReplaceDialogControl(m_hWnd, IDC_CORNER_VIEW_PLACEHOLDER)) return FALSE;
  m_MainViewport.SetBackgroundColor(0.9f, 0.9f, 0.9f);
  m_CornerViewport.SetBackgroundColor(0.9f, 0.9f, 0.9f);

  
  //A Window that can contain viewport objects exists. We need to attach an IViewport
  //to the window for handling drawing.
  //Initialize the Direct3D Device
  if(!m_MainUIWindow.AttachViewport(&m_MainViewport)) return FALSE;
  if(!m_CornerUIWindow.AttachViewport(&m_CornerViewport)) return FALSE;

  InitializeCameras();

  m_CornerViewport.SetOtherCamera( &(m_MainViewport.m_Camera) );
 
  m_CameraControl.Create(IDD_CAMERA_CONTROL, this);
  m_CameraControl.ShowWindow(SW_HIDE);
  m_CameraControl.SetCameras(&(m_MainViewport.m_Camera), &(m_CornerViewport.m_Camera));

  m_LightControl.Create(IDD_LIGHT_CONTROL, this);
  m_LightControl.ShowWindow(SW_HIDE);

  m_ShowCameraControl = FALSE;

  CWnd* pMsgWnd = GetDlgItem(IDC_STATUS_AREA);
  if(pMsgWnd) g_hMessageWindow = pMsgWnd->m_hWnd;

  EchoToStatusArea("Dialog::OnInitDialog completed");

  // Here we initialize the timer, to go off every 25 milli seconds
  // when timer "goes-off", our OnTimer() function will be called and
  // it is upto us to decide what we want to do.
  // our goal is to approx 40 frames per second (25 millisecond refresh rate)
  SetTimer(0, 25, NULL);  
	
  UpdateData(FALSE);  // set all states to the control from internal variables

  return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void ClassExampleDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this); // device context for painting
    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);
    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;
    // Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  } else {
    CDialog::OnPaint();
  }
}

// The system calls this function to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR ClassExampleDlg::OnQueryDragIcon()
{
  return static_cast<HCURSOR>(m_hIcon);
}

void ClassExampleDlg::OnTimer(UINT nIDEvent)
{
  theWorld.UpdateWorld();
  m_MainUIWindow.DrawViewport();
  m_CornerUIWindow.DrawViewport();
}

void ClassExampleDlg::InitializeCameras()
{
  
  #define kInitCameraFOV  M_PI/4.0

  m_MainViewport.m_Camera.Set(D3DXVECTOR3(0,15,25), //eye
                              D3DXVECTOR3(0, 0, 0), //at
                              D3DXVECTOR3(0, 1, 0), //up
                              kInitCameraFOV, // fov: PI/4 (45-degree) field of view
                              1.0f,           // near clipping plane
                              100.0f);        // far clipping plane

  m_CornerViewport.m_Camera.Set(D3DXVECTOR3(15,15,15), //eye
                                D3DXVECTOR3(0, 0, 0), //at
                                D3DXVECTOR3(0, 1, 0), //up
                                kInitCameraFOV, // fov: PI/4 (45-degree) field of view
                                1.0f,           // near clipping plane
                                200.0f);        // far clipping plane
}
void ClassExampleDlg::OnBnClickedResetWorld()
{
  UpdateData(FALSE);

  InitializeCameras();

  EchoToStatusArea("Reset everyting");
}


void ClassExampleDlg::OnBnClickedShowCameralControl()
{
  UpdateData(TRUE);
  if (m_ShowCameraControl) {
      m_CameraControl.ShowWindow(SW_SHOW);
  } else {
      m_CameraControl.ShowWindow(SW_HIDE);
  }
}

void ClassExampleDlg::OnBnClickedShowLightControl()
{
  UpdateData(TRUE);
  if (m_ShowLightControl) {
      m_LightControl.ShowWindow(SW_SHOW);
  } else {
      m_LightControl.ShowWindow(SW_HIDE);
  }
}

void ClassExampleDlg::OnBnClickedTriangleOrientation()
{
  UpdateData(TRUE);
  theWorld.m_DrawTriangleOrientation = m_TriangleProblem ? true : false;
}

void ClassExampleDlg::OnBnClickedPerspectiveDistortion()
{
  UpdateData(TRUE);
  theWorld.m_DrawDistortion = this->m_PerspectiveProblem ? true : false;
}

void ClassExampleDlg::OnBnClickedSilhouette()
{
  UpdateData(TRUE);
  theWorld.m_DrawSilhouette = this->m_SilhouetteProblem ? true : false;
}

void ClassExampleDlg::OnBnClickedNormalProblem()
{
  UpdateData(TRUE);
  theWorld.m_DrawNormalProblem = this->m_NormalProblem ? true : false;
}

void ClassExampleDlg::OnBnClickedTJunction()
{
  UpdateData(TRUE);
  theWorld.m_DrawTjunction = this->m_TJunctionProblem ? true : false;
}