// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_D3DDefines.h"
#include "../uwbgl_Valuator.h"

namespace UWBGL_D3D
{
  class Camera
  {
  public:
    void Set(const D3DXVECTOR3& eye, const D3DXVECTOR3& at, const D3DXVECTOR3& up, float fov, float near_clip, float far_clip);
    void SetView(const D3DXVECTOR3& eye, const D3DXVECTOR3& at, const D3DXVECTOR3& up);

    void DefineViewAndProjection(float aspectRatio, LPDIRECT3DDEVICE9 pDevice) const;
    void Draw(); //draw camera
    void DrawRailRepresentation(LPDIRECT3DDEVICE9 device);
    
    float GetViewDistance() const;
    void SetViewDistance(float distance, bool move_eye_position);

    void SetFieldOfView(float value){m_FieldOfView = value;}
    float GetFieldOfView() const {return m_FieldOfView;}

    //mouse rotation
    void BeginMouseRotate(int x, int y);
    void MouseRotate(int x, int y);

    //mouse tracking
    void BeginMouseTracking(int x, int y);
    void MouseTrack(int x, int y);

    //mouse zoom
    void BeginMouseZoom(int x);
    void MouseZoom(int x, bool move_eye);
  private:
    Camera();
    virtual ~Camera();
    friend class D3DViewport; //this way only a D3DViewport can create a Camera object

    void ComputeCameraFrame(float &d, D3DXVECTOR3&, D3DXVECTOR3&, D3DXVECTOR3&);
    void RotateCameraX(float);
    void RotateCameraY(float);

    D3DXVECTOR3 m_EyePosition;
    D3DXVECTOR3 m_AtPosition;
    D3DXVECTOR3 m_UpVector;
    float m_FieldOfView;
    float m_Near;
    float m_Far;

    bool m_bCameraVisible;

    //rotation
    Valuator m_DeltaXRot, m_DeltaYRot;
    Valuator m_DeltaXTrack, m_DeltaYTrack;
    Valuator m_DeltaZoom;
  };
};