#include "uwbgl_D3DCamera.h"
#include "uwbgl_D3DGraphicsSystem.h"


using namespace UWBGL_D3D;

void Camera::SetViewDistance(float distance, bool move_eye_position)
{
  if(distance < 0.1f)
    distance = 0.1f;  // this is very small, anything smaller does not make sense anymore!

  D3DXVECTOR3 unit_view_vector = m_AtPosition - m_EyePosition;
  D3DXVec3Normalize(&unit_view_vector, &unit_view_vector);

  if(move_eye_position)
    m_EyePosition = m_AtPosition - (distance*unit_view_vector);
  else
    m_AtPosition = m_EyePosition + (distance*unit_view_vector);
}


#define ROTATE_ANGLE_RANGE        80.0f
#define ROTATE_MOUSE_MOVE_RANGE   200.0f
#define MIN_DELTA_ROTATE          (D3DXToRadian(0.1f))

void Camera::BeginMouseRotate(int x, int y)
{
  m_DeltaXRot.InitializeValuator(y-ROTATE_MOUSE_MOVE_RANGE, y+ROTATE_MOUSE_MOVE_RANGE, -ROTATE_ANGLE_RANGE, ROTATE_ANGLE_RANGE);
  m_DeltaYRot.InitializeValuator(x-ROTATE_MOUSE_MOVE_RANGE, x+ROTATE_MOUSE_MOVE_RANGE, -ROTATE_ANGLE_RANGE, ROTATE_ANGLE_RANGE);
}

void Camera::MouseRotate(int x, int y)
{
  float dx = -m_DeltaXRot.GetScaleValue(float(y))*M_PI/180.0f;
  float dy = -m_DeltaYRot.GetScaleValue(float(x))*M_PI/180.0f;

  if (fabs(dy) > MIN_DELTA_ROTATE) {
    RotateCameraY(dy);
    m_DeltaYRot.InitializeValuator(x-ROTATE_MOUSE_MOVE_RANGE, x+ROTATE_MOUSE_MOVE_RANGE, -ROTATE_ANGLE_RANGE, ROTATE_ANGLE_RANGE); 
  }

  if (fabs(dx) > MIN_DELTA_ROTATE) {
    RotateCameraX(dx);
    m_DeltaXRot.InitializeValuator(y-ROTATE_MOUSE_MOVE_RANGE, y+ROTATE_MOUSE_MOVE_RANGE, -ROTATE_ANGLE_RANGE, ROTATE_ANGLE_RANGE);
  }
}

// Constants for tracking the camera
#define kTrackingDelta 100
#define kTrackingSize   10

// Constants for zooming the camera
#define kZoomingDelta  100
#define kZoomingSize   5.0f


void Camera::BeginMouseTracking(int x, int y)
{
  m_DeltaXTrack.InitializeValuator((float)(x-kTrackingDelta), (float)(x+kTrackingDelta), -kTrackingSize, kTrackingSize);
  m_DeltaYTrack.InitializeValuator((float)(y-kTrackingDelta), (float)(y+kTrackingDelta), -kTrackingSize, kTrackingSize);
}

void Camera::MouseTrack(int x, int y)
{
  float dx = m_DeltaXTrack.GetScaleValue((float)x);
  float dy = m_DeltaYTrack.GetScaleValue((float)y);

  float       d;
  D3DXVECTOR3 u, v, w;
  this->ComputeCameraFrame(d, v, u, w);

  m_EyePosition = m_EyePosition + (w* dx) + (u * dy);
  m_AtPosition = m_AtPosition + (w * dx) + (u * dy);
  m_DeltaXTrack.InitializeValuator((float)(x-kTrackingDelta), (float)(x+kTrackingDelta), -kTrackingSize, kTrackingSize);
  m_DeltaYTrack.InitializeValuator((float)(y-kTrackingDelta), (float)(y+kTrackingDelta), -kTrackingSize, kTrackingSize);
}

void Camera::BeginMouseZoom(int x)
{
  m_DeltaZoom.InitializeValuator((float)(x-kZoomingDelta), (float)(x+kZoomingDelta), -kZoomingSize, kZoomingSize);
}

void Camera::MouseZoom(int x, bool move_eye)
{
  float dx = m_DeltaZoom.GetScaleValue((float)x);
  float d = GetViewDistance();
  SetViewDistance(GetViewDistance() + dx, move_eye);
  m_DeltaZoom.InitializeValuator((float)(x-kZoomingDelta), (float)(x+kZoomingDelta), -kZoomingSize, kZoomingSize);
}