#include "uwbgl_D3DDirectionalLight.h"
#include "uwbgl_D3DArrowSceneNode.h"

using namespace UWBGL_D3D;

DirectionalLight::DirectionalLight(int id) : Light(id)
{
  SetToDirectionalLight();
}

DirectionalLight::~DirectionalLight()
{
}

void DirectionalLight::DrawLightGeometry()
{
  // We will draw directional light with the arrow. 
  // now, we remember the arrow points towards the +ve y-axis
  // we need a rotation that will rotate y-axis toward m_d3dLight.Direction
  D3DXVECTOR3 d(m_d3dLight.Direction), y(0.0f, 1.0f, 0.0f);
  D3DXVec3Normalize(&d, &d);

  float cosTheta = D3DXVec3Dot(&d, &y);
  float theta = acosf( cosTheta );
  
  D3DXVECTOR3 rAxis(1.0f, 0.0f, 0.0f);
  if (fabs(cosTheta) > 0.999f) {
        // almost on the same line ...
        if (cosTheta < 0.0f) {
              // pointing in the opposite direciont
              theta = D3DX_PI;
        } else {
              // point in the same direction, we are done
              theta = 0.0f;
        }
  } else {

        D3DXVec3Cross(&rAxis, &y, &d);
  }
  
  D3DXQUATERNION q;
  D3DXQuaternionRotationAxis(&q, &rAxis, theta);
  
  D3DXCOLOR c;
  GetDiffuse(c);

  ArrowSceneNode a;
  a.SetArrowColor( c, c );
  a.m_XForm.SetTranslationTo( m_d3dLight.Position );
  a.m_XForm.SetScaleTo(1.2f, 1.0f, 1.2f);
  a.m_XForm.SetOrientationQ(q);
  a.DrawNode();
}

