#pragma once
#include "uwbgl_D3DDefines.h"
#include "uwbgl_D3DPrimitiveBase.h"
#include "uwbgl_D3DMaterial.h"

namespace UWBGL_D3D 
{
  class DrawablePrimitive : public PrimitiveBase
  {
    // D3DXVECTOR3 m_Velocity (speed + direction)
    CompoundPrimitiveAttr(D3DXVECTOR3, Velocity)

    // D3DXCOLOR m_DrawColor (color to be drawn)
    CompoundPrimitiveAttr(D3DXCOLOR, DrawColor)

    // bool m_Texture (Is texture mapping switched on?)
    BoolPrimitiveAttr(Texture)

    // int m_TextureIndex (if texture mapping is on, this is the index to Viewport texture table)
    SimplePrimitiveAttr(int, TextureIndex)

    // bool m_Blending (should we perform alpha blending?)
    BoolPrimitiveAttr(Blending)

    // Material m_Material (what is the material for this object)
    CompoundPrimitiveAttr(Material, Material)

    // bool m_Lighting (is lighting enabled?)
    BoolPrimitiveAttr(Lighting)
  protected:
    // std::vector<D3DXVECTOR3>	m_Vertices
    //    or (D3DVertexList as defined in uwbgl_D3DDefines.h")
    D3DVertexList m_Vertices;		

    // Following methods are for subclass to override
    // which will define/modify the detailed behavior
    // of each types of primitive
    virtual void SetPrimitiveAttributesForDrawing();
    // subclass is responsible for setting the primitive's attributes

    virtual void DrawPrimitiveGeometry() = 0;
    // the drawing of just the geometric informaiton of the primitive

  public:
    virtual ~DrawablePrimitive();
    DrawablePrimitive();

    void DrawPrimitive();
    // Draws the primitive, with the corresponding geometry and correct attributes

    // from PrimitiveBase
    void GetColliderList(ColliderList &list);

    //  Support for the VertexList ..
    int	 NumVertices() const;
    const D3DVertexList& GetVertices() const;
    virtual void SetFirstVertex(const D3DXVECTOR3 &p);
    virtual void AppendVertex(const D3DXVECTOR3 &v);
    virtual void ReplaceVertexAt(int index, const D3DXVECTOR3 &v);
    virtual const D3DXVECTOR3& GetVertexAt(int index) const;
    
    virtual void GetPrimitiveCentroid(D3DXVECTOR3&);
    // Center of gravity of the primitive

    virtual void MovePrimitiveByVelocity();
    //  Updates location by m_Velocity

    virtual bool OutOfBound(float minx, float miny, float maxx, float maxy);
    // returns true if primitive is out of bound, default behavior is to
    // test if vertex[0] is out of bound. If there are no vertices
    // returns false;

    virtual void MovePrimitiveTo(const D3DXVECTOR3 &d) = 0;
    // Moves primitive to this new location
    // Individual primitive has to decide what this means.

    // the following set of behaviors allows DrawablePrimitive understand
    // the language of all subclasses.  This is important for
    // "generic" graphics operation, e.g. selection in PrimitiveList

    //  point size
    SetGetAttrFuncNames(float, PointSize, )

    // shadable attrbiute
    SetGetAttrFuncNames(WORD, ShadedMode, )

    // fillable attribute
    SetGetAttrFuncNames(WORD, FillMode, )
  };
};
