#include "../uwbgl_Defines.h"
#include "uwbgl_D3DFileTexture.h"

using namespace UWBGL_D3D;

FileTexture::~FileTexture()
{
	if (m_Texture != NULL)
		m_Texture->Release();
}

FileTexture::FileTexture(LPDIRECT3DDEVICE9 d, const char* filename)
{
	m_Texture = NULL;
	m_D3dDevice = d;
	CreateD3DTexture(d, filename);
}


BOOL FileTexture::CreateD3DTexture(LPDIRECT3DDEVICE9 pd3dDevice, const char* filename)
{
	HRESULT hr;
	if( FAILED( hr = D3DXCreateTextureFromFile(m_D3dDevice, filename, &m_Texture ) ) )
	{
		return FALSE;
	}
	return TRUE;
}

void FileTexture::ActivateTexture(DWORD stage)
{
	if (m_Texture == NULL)
		return;

  	// set up our texture stages for a simple texture copy...
		m_D3dDevice->SetTextureStageState( stage, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
				// 
				// Refer to the D3D help, the first parameter is a number (between 0 to something)
				//
				// the second parameter can be ...
				//				D3DTSS_COLOROP
				//				D3DTSS_COLORARG1, ARG2
				//				D3DTSS_ALPHAARG1, ARG2
				//				D3DTSS_MAGFILTER, MINFILTER, MIPFILTER (more and more and tons tons more!!)
				//
				// For Stage 0,
				//
				//		Arg1 is the texture
				//		Arg2 is the color on the object

				// The third parameter can be
				//				D3DTA_TEXTURE
				//				D3DTA_DIFFUSE
				//				D3DTA_MODULATE
				//				D3DTA_SELECTARG1
				//				D3DTA_SELECTARG2 ... (more and more and tons tons more!!!)

		m_D3dDevice->SetTextureStageState( stage, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
				// Now see, are setting the alpha channel in the texture stage, in this
				// case, we say, we _always_ want the alpha information from the file texture
				//

		m_D3dDevice->SetTexture(stage, m_Texture);
}

void FileTexture::DeActivateTexture(DWORD stage)
{
	//
	// Is this the right way to deactivate texture?
	//
	m_D3dDevice->SetTexture(stage, NULL);

	//
	// I think this is a better way to disable texture?
	m_D3dDevice->SetTextureStageState( stage, D3DTSS_COLOROP,   D3DTOP_DISABLE);
}
