#include "uwbgl_D3DGraphicsSystem.h"

using namespace UWBGL_D3D;

GraphicsSystem GraphicsSystem::m_Graphics;

bool GraphicsSystem::Initialize()
{
  if(m_Graphics.m_pD3D==NULL)
  {
    m_Graphics.m_pD3D = Direct3DCreate9( D3D_SDK_VERSION);
  }
  if(m_Graphics.m_pD3D==NULL) return false;

  // Now create our matrix stack to help us transform from 
  // object space to world space
  if( FAILED( D3DXCreateMatrixStack(0, &(m_Graphics.m_pObj2WorldStack))))
  {
    //MessageBox("D3d Matrix Stack Creation Failed!");
    return false;
  }
  if( FAILED( m_Graphics.m_pObj2WorldStack->LoadIdentity())) 
  {
    //MessageBox("D3d Matrix Stack Init to Identity failed!");
    return false;
  }
  return true;
}

void GraphicsSystem::Terminate()
{
  m_Graphics.SetActiveDevice(NULL);
  if(m_Graphics.m_pD3D)
  {
    m_Graphics.m_pD3D->Release();
    m_Graphics.m_pD3D = NULL;
  }
  if(m_Graphics.m_pObj2WorldStack)
  {
    m_Graphics.m_pObj2WorldStack->Release();
    m_Graphics.m_pObj2WorldStack = NULL;
  }
}

GraphicsSystem::GraphicsSystem() :
  m_pD3D(NULL),
  m_pActiveDevice(NULL),
  m_pObj2WorldStack(NULL)
{
}

GraphicsSystem::~GraphicsSystem()
{
  Terminate();
}

void GraphicsSystem::SetActiveDevice(LPDIRECT3DDEVICE9 pDevice)
{
  if(m_pActiveDevice) m_pActiveDevice->Release();
  m_pActiveDevice = pDevice;
  if(m_pActiveDevice) m_pActiveDevice->AddRef();
}
bool GraphicsSystem::GraphicsIsInitialized()
{
  return m_pD3D != NULL;
}
LPDIRECT3DDEVICE9 GraphicsSystem::GetActiveDevice()
{
  return m_pActiveDevice;
}
