// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_D3DDefines.h"

#define SetLightColor(materialType)           \
  void Set##materialType##(float v){          \
    m_d3dLight.##materialType##.r = v;       \
    m_d3dLight.##materialType##.g = v;       \
    m_d3dLight.##materialType##.b = v;       \
    m_d3dLight.##materialType##.a = 1.0;     \
  }                                          \
  void Set##materialType##(float r, float g, float b, float a=1.0) {  \
    m_d3dLight.##materialType##.r = r;       \
    m_d3dLight.##materialType##.g = g;       \
    m_d3dLight.##materialType##.b = b;       \
    m_d3dLight.##materialType##.a = a;       \
  }                                          \
  void Set##materialType##(const D3DXCOLOR &v) {  \
    m_d3dLight.##materialType##.r = v.r;     \
    m_d3dLight.##materialType##.g = v.g;     \
    m_d3dLight.##materialType##.b = v.b;     \
    m_d3dLight.##materialType##.a = v.a;     \
  }                                          \
  void Get##materialType##(D3DXCOLOR &v) const {  \
    v.r = m_d3dLight.##materialType##.r;     \
    v.g = m_d3dLight.##materialType##.g;     \
    v.b = m_d3dLight.##materialType##.b;     \
    v.a = m_d3dLight.##materialType##.a;     \
  }

#define SetLightAttr(_attr_)                         \
  void Set##_attr_##(float v) { m_d3dLight.##_attr_## = v;} \
  float Get##_attr_##() { return m_d3dLight.##_attr_##; }

namespace UWBGL_D3D
{
  // light information for D3DX: We wrap around D3DLIGHT9 structure providing a collection
  // of accessing methods.
  class Light
  {
  public:
    Light(int id);
    virtual ~Light();

    void          DrawLight();

    D3DLIGHTTYPE  GetLightType() { return m_d3dLight.Type; }

    // define the setting and accessing functions
    SetLightColor(Ambient)
    SetLightColor(Diffuse)
    SetLightColor(Specular)
    SetLightAttr(Range)
    SetLightAttr(Falloff)
    SetLightAttr(Attenuation0)
    SetLightAttr(Attenuation1)
    SetLightAttr(Attenuation2)
    SetLightAttr(Theta)
    SetLightAttr(Phi)

    void SetPosition(float x, float y, float z);
    void SetPosition(const D3DXVECTOR3 &v);
    void GetPosition(D3DXVECTOR3 &v) const;

    void SetDirection(float x, float y, float z);
    void SetDirection(const D3DXVECTOR3 &v);
    void GetDirection(D3DXVECTOR3 &v) const;

    void SwitchOnLight();
    void SwitchOffLight();
    bool LightIsSwitchedOn() const { return m_bLightIsOn; }

    void EnableDrawLight() { m_bDrawLight= true; }
    void DisableDrawLight() {m_bDrawLight = false; }
    bool ShouldDrawLight() { return m_bDrawLight; }

    void SetLightToDevice() const;

  protected:

    // Pure virtual function that must be override!!
    virtual void DrawLightGeometry() = 0;

    void SetToSpotLight() { m_d3dLight.Type = D3DLIGHT_SPOT; }
    void SetToPointLight() { m_d3dLight.Type = D3DLIGHT_POINT; }
    void SetToDirectionalLight() { m_d3dLight.Type = D3DLIGHT_DIRECTIONAL; }


    bool m_bLightIsOn;
    bool m_bDrawLight;

    // There are about 8-16 hardware light sources. This is why we hold an
    // internal array of lights and provide access through a static function
    int m_LightID;

    D3DLIGHT9 m_d3dLight; // this is the light
    // has: Type      (D3DLIGHT_POINT, DIRECTIONAL, or SPOT)
    //      Ambient, Diffuse, Specular   (D3DCOLORVALUE)
    //      Position, Direction          (D3DVECTOR)
    //      Range, Falloff               float
    //      Attenuation 0, 1, 2
    //      Theta     (for spot light)
    //      Phi
  };
};
