#include "uwbgl_D3DGraphicsSystem.h"
#include "uwbgl_D3DDeviceVertexFormat.h"
#include "uwbgl_D3DLinePrimitive.h"

using namespace UWBGL_D3D;

LinePrimitive::LinePrimitive()
{}

LinePrimitive::~LinePrimitive() 
{}

void 
LinePrimitive::AppendVertex(const D3DXVECTOR3 &p)
{
  if (NumVertices() > 1)
    m_Vertices[1] = p;
  else
    DrawablePrimitive::AppendVertex(p);
}
void LinePrimitive::SetFirstVertex(const D3DXVECTOR3 &p)
{
  DrawablePrimitive::SetFirstVertex(p);
  if (NumVertices() > 1)
    ReplaceVertexAt(1, p);
}

void LinePrimitive::MovePrimitiveTo(const D3DXVECTOR3 &pt)
{
  // We define moving of line as centered around the mid position 
  // of the line
  D3DXVECTOR3 mid(0.0f, 0.0f, 0.0f);
  GetPrimitiveCentroid(mid);

  D3DXVECTOR3 delta = pt - mid;
  for (unsigned int i=0; i<m_Vertices.size(); i++)
    m_Vertices[i] += delta;
}


void LinePrimitive::DrawPrimitiveGeometry()
{
  if (NumVertices() == 0)
    return;

  LPDIRECT3DDEVICE9 d3dDevice = GraphicsSystem::GetSystem().GetActiveDevice();
  if (d3dDevice == NULL) return;

  DeviceVertexFormat v[2];
  v[0].m_Point = m_Vertices[0];
  v[0].m_Color = D3DCOLOR_COLORVALUE(m_DrawColor.r, m_DrawColor.g, m_DrawColor.b, m_DrawColor.a);

  v[1] = v[0];
  if (NumVertices() > 1)
    v[1].m_Point = m_Vertices[1];

  if ( GetShadedMode() == D3DSHADE_GOURAUD)
    v[1].m_Color = D3DCOLOR_COLORVALUE(0.9f, 0.9f, 0.9f, 0.9f);

  d3dDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, (CONST void *)v, sizeof(DeviceVertexFormat));
}

void LinePrimitive::SetEnds(float pt0_x, float pt0_y, float pt0_z, float pt1_x, float pt1_y, float pt1_z)
{
  SetFirstVertex(D3DXVECTOR3(pt0_x,pt0_y,pt0_z));
  AppendVertex(D3DXVECTOR3(pt1_x,pt1_y,pt1_z));
}
