#include "uwbgl_D3DDeviceVertexFormat.h"
#include "uwbgl_D3DPointPrimitive.h"
#include "uwbgl_D3DCollider.h"
#include "uwbgl_D3DGraphicsSystem.h"

using namespace UWBGL_D3D;

ImplementSimplePrimitiveAttr(PointPrimitive, float, PointSize)

PointPrimitive::PointPrimitive() : 
	m_PointSize(1.0f)
{
}

PointPrimitive::~PointPrimitive() 
{
}

void PointPrimitive::AppendVertex(const D3DXVECTOR3 &pt)
{
  if (NumVertices() == 1)
		m_Vertices[0] = pt;
	else
		DrawablePrimitive::AppendVertex(pt);
}

void PointPrimitive::MovePrimitiveTo(const D3DXVECTOR3 &pt)
{
	if (NumVertices() == 0)
		AppendVertex(pt);
	else
		ReplaceVertexAt(0, pt);
}


void PointPrimitive::SetPrimitiveAttributesForDrawing() 
{
	LPDIRECT3DDEVICE9 device = GraphicsSystem::GetSystem().GetActiveDevice();
	if (device == NULL) return;

	//
	// Calls the parent class to do its work
	DrawablePrimitive::SetPrimitiveAttributesForDrawing();
	device->SetRenderState(D3DRS_POINTSIZE, *((DWORD*)(&m_PointSize)));
}

void PointPrimitive::DrawPrimitiveGeometry()
{
	LPDIRECT3DDEVICE9 device = GraphicsSystem::GetSystem().GetActiveDevice();
	if (device == NULL) return;

  if (NumVertices() == 0)
    return;

	DeviceVertexFormat v;
	v.m_Point = m_Vertices[0];
	v.m_Color = D3DCOLOR_COLORVALUE(m_DrawColor.r, m_DrawColor.g, m_DrawColor.b, m_DrawColor.a);
	device->DrawPrimitiveUP(D3DPT_POINTLIST, 1, (CONST void *)(&v), sizeof(DeviceVertexFormat));
}

void PointPrimitive::GetColliderList(ColliderList &l)
{
  if (NumVertices() > 0) {
    Collider c(m_Vertices[0], m_PointSize/2.0f);
	  l.Append(c);
  }
}
