#pragma once
#include "uwbgl_D3DDefines.h"

// Use /P option in preprocessor command line option to generate a .i file
// so that you can examine the output from pre-processor 
//
// (for IDE .net 2003) Project->Properties->C/C++-> Command Line 
// Additional Options: /P.
//
// Note with the /P optoin, only expended header will be generated
// while _NO_ compilation will happen!! After "compiling" is done,
// go look for ".i" file in your working directory.


#define SetGetAttrFuncNames(_dataType_, _attrName_, _def_)    \
  virtual void Set##_attrName_##(const _dataType_ &na) _def_; \
  virtual _dataType_##& Get##_attrName_##() _def_;

#define BoolAttrFuncNames(_attrName_, _def_)  \
  virtual void Enable##_attrName_##() _def_;  \
  virtual void Disable##_attrName_##() _def_; \
  virtual bool  _attrName_##IsEnabled() const _def_;


// Set and get attr/function for compound data type:
//  	e.g. D3DXCOLOR
#define CompoundPrimitiveAttr(_dataType_, _attrName_) \
  protected:                                          \
    _dataType_  m_##_attrName_##;                     \
  public:                                             \
    SetGetAttrFuncNames(_dataType_, _attrName_, )     \


// For each defined CompoundPrimitiveAttr, the following macro should
// be placed in the .cpp file to implement the basic Set and Get
// function for the attribute
#define ImplementCompoundPrimitiveAttr(_className_, _dataType_, _attrName_)               \
  void _className_##::Set##_attrName_##(const _dataType_ &na) { m_##_attrName_## = na; }  \
  _dataType_##& ##_className_##::Get##_attrName_##() { return m_##_attrName_##; }

// Set and get attr/function for simple data type:
//  	e.g. D3DXCOLOR (this is a DWORD)
//  	   or int,
//  	   or float, etc.
#define SimplePrimitiveAttr(_dataType_, _attrName_) \
  protected:                                        \
    _dataType_  m_##_attrName_##;                   \
  public:                                           \
    SetGetAttrFuncNames(_dataType_, _attrName_, )
			
#define ImplementSimplePrimitiveAttr(_className_, _dataType_, _attrName_)                   \
  void _className_##::Set##_attrName_##(const _dataType_##& na) { m_##_attrName_## = na; }  \
  _dataType_##&  _className_##::Get##_attrName_##() { return m_##_attrName_##; }

// Set and get attr/function for boolean data type:
//  	e.g. bool m_Visiblility
#define BoolPrimitiveAttr(_attrName_) \
  protected:                          \
    bool m_##_attrName_##;            \
  public:                             \
    BoolAttrFuncNames(_attrName_, )

#define ImplementBoolPrimitiveAttr(_className_, _attrName_)                     \
  void _className_##::Enable##_attrName_##() { m_##_attrName_## = true; }       \
  void _className_##::Disable##_attrName_##() { m_##_attrName_## = false; }     \
  bool _className_##::_attrName_##IsEnabled() const { return m_##_attrName_##; }


namespace UWBGL_D3D
{
  class SceneNode;
  class ColliderList;
  
  class PrimitiveBase
  {
    // bool m_Visibility; (is this primitive visible?)
    BoolPrimitiveAttr(Visibility)

    // bool m_ShowColliders (should we draw our collider?)
    BoolPrimitiveAttr(ShowColliders)

  protected:
    // should this primitive be drawn (e.g. is it visible?)
    virtual bool ShouldDrawPrimitive();
  public:
    virtual ~PrimitiveBase();
    PrimitiveBase();

    virtual void DrawColliders();

    // returns a list of collider for this primitive.
    // subclass is expected to compute the colliders 
    // and construct this list at run time.
    virtual  void GetColliderList(ColliderList&) = 0;

    virtual void DrawPrimitive() = 0;

    void SetParentNode(SceneNode* pSceneNode);
  protected:
    SceneNode* m_pParentSceneNode;
  };
};
