#include "uwbgl_D3DViewport.h"
#include "uwbgl_D3DGraphicsSystem.h"
#include "uwbgl_D3DFileTexture.h"
#include "uwbgl_D3DRawMesh.h"

using namespace UWBGL_D3D;


static const char *FileTextureNames[] = {
                        "TextureFiles/CMMR.png",              // will locate at entry 0
                        "TextureFiles/KelvinInOffice.png",    // will locate at entry 1 ...
                        "TextureFiles/binoculars.jpg",
                        "TextureFiles/Building1.jpg",
                        "TextureFiles/Building2.jpg",
                        "TextureFiles/CSSCover.jpg",
                        "TextureFiles/edgy_promenade.jpg",
                        NULL};


static const int kMeshIndexOffset = 100;
          // 99 and lower is file textures
          // 100 and above is mesh

static const char *MeshFolder = "./MeshFiles";
static const char *MeshNames[] = {
                        "cylinder.x",           // remember, eCylinderMesh is 100 (defined in D3DDefines.h)
                        "cone.x",               // eConeMesh   is 101
                        "sphere.x",             // eSphereMesh is 102
                        "box.x",                // eBoxMesh    is 103
                        "car.x",                // eCarMesh    is 104
                        "shusui.x",             // eShuSuiMesh is 105
                        "bigship1.x",           // eBigShip    is 106
                        "tiger.x",              // eTigerMesh  is 107
                        "teapot.x",             // eTeapotMesh is 108
                        "HiResFloor.x",         // eHiResMesh  is 109
                        "OpenCone.x",           // eOpenConeMesh is 110
                        NULL};

//
// All the methods supporting initialization of File Texture are here.
//
void D3DViewport::InitializeResourceTable()
{

    int i=0, j=0;


    // if we are re-initializing this table, make sure we release all existing entries
    ClearResourceTable();
    

    // now load file textures to 0 to however many file textures we have (MUST BE less than 100)
    i = 0;
    while (FileTextureNames[i] != NULL)  {
          FileTexture *f = new FileTexture(m_pD3DDevice, FileTextureNames[i]);
          m_ResourceTable.push_back( (void *) f );
          i++;
    }
    // for all entries less than 100, fill with NULL
    for (j=i; j<kMeshIndexOffset; j++)
          m_ResourceTable.push_back( NULL );

    // now load mesh files ...
    j = 0;
    while (MeshNames[j] != NULL) {
          RawMesh *m = new RawMesh();
          m->LoadMesh(m_pD3DDevice, GraphicsSystem::GetSystem().GetWorldStack(), MeshFolder, MeshNames[j]);
          m_ResourceTable.push_back( (void *) m);
          j++;
    }
}


void D3DViewport::ClearResourceTable()
{
    int i;
    for (i=(int)(m_ResourceTable.size()-1);  i>=0; i--) {
        if (i >= kMeshIndexOffset) {
            RawMesh *m = (RawMesh *) m_ResourceTable[i];
            if (m != NULL)
                delete m;
        } else {
            FileTexture *f = (FileTexture *) m_ResourceTable[i];
            if (f != NULL)
                delete f;
        }
        std::vector<void*>::iterator it = m_ResourceTable.begin() + i;
        m_ResourceTable.erase(it);
    }
}
