// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "uwbgl_D3DDefines.h"

namespace UWBGL_D3D
{
  class XFormInfo
  {
  public:
    XFormInfo();

    void ResetTransformation();

    void SetUpD3DXformStack(ID3DXMatrixStack *stack) const;

    // accessing the pivot
    const D3DXVECTOR3& XFormPivot() const;
    void SetPivotTo(const D3DXVECTOR3 &p);
    void SetPivotTo(float x, float y, float z);
		void SetPivotXTo(float x);
		void SetPivotYTo(float y);
		void SetPivotZTo(float z);

    // accessing the xform info 
    float TranslateX() const;
    float TranslateY() const;
    float TranslateZ() const;
    const D3DXVECTOR3& Translation() const;

    float ScaleX() const;
    float ScaleY() const;
    float ScaleZ() const;
    const D3DXVECTOR3& Scale() const;

    void GetOrientation(D3DXMATRIX &m) const;
    void GetOrientationQ(D3DXQUATERNION &q) const;
		
    	
    // Setting the xform info
    // SetXXTo() - funcitons, _sets_ the values of the xform info
    // UpdateXXBy() - funcitons _modifies_ the xform info values by the input
    //  Movement - adds fMove to the inpit
    //  Scale    - multiplies fScale to the input
    //  Rotate   - adds fRotation to the input.
    //
    void SetTranslationTo(const D3DXVECTOR3 &d);
    void SetTranslationTo(float x, float y, float z);
		void SetTranslationXTo(float x);
		void SetTranslationYTo(float y);
		void SetTranslationZTo(float z);
    void UpdateTranslationBy(const D3DXVECTOR3 &v);
    void UpdateTranslationBy(float x, float y, float z);
		void UpdateTranslationXBy(float x);
		void UpdateTranslationYBy(float y);
		void UpdateTranslationZBy(float z);

    void SetScaleTo(const D3DXVECTOR3 &s);
    void SetScaleTo(float s);
    void SetScaleTo(float x, float y, float z);
		void SetScaleXTo(float x);
		void SetScaleYTo(float y);
		void SetScaleZTo(float z);
    void UpdateScaleBy(float s);
    void UpdateScaleBy(float x, float y, float z);
		void UpdateScaleXBy(float x);
		void UpdateScaleYBy(float y);
		void UpdateScaleZBy(float z);

    
    
    void SetRotationToDegree(float x, float y, float z);
    void SetRotationToRadian(float x, float y, float z);
    void SetRotationToDegree(D3DXVECTOR3 &r);
    void SetRotationToRadian(D3DXVECTOR3 &r);
		void SetOrientation(const D3DXMATRIX  &m);
    void SetOrientationQ(const D3DXQUATERNION &q);
		
	  void UpdateRotationXByRadian(float x);
		void UpdateRotationXByDegree(float x);
		void UpdateRotationYByRadian(float y);
		void UpdateRotationYByDegree(float y);
		void UpdateRotationZByRadian(float z);
		void UpdateRotationZByDegree(float z);
    void UpdateRotationBy(const D3DXQUATERNION &q);
    void UpdateRotationBy(const D3DXMATRIX &m);

  private:
    D3DXVECTOR3       m_Translate;		
    D3DXVECTOR3       m_Scale;	
    D3DXQUATERNION    m_Rotation;   // results of rotations
    D3DXVECTOR3       m_Pivot;      // pivot position for rotation and scaling.
  };
};

