// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#include "uwbgl_MFC.h"
#include "uwbgl_MFCSliderCtrlWithEcho.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(SliderCtrlWithEcho, CSliderCtrl)

/// Class constructor initializes all class members to default values:
/// - fMinRange == 0.0f, fTotalRange == 1.0f, m_bIsVert == false
SliderCtrlWithEcho::SliderCtrlWithEcho() :
  fMinRange(0.0f),
  fTotalRange(1.0f),
  m_bIsVert(false)
{
}

SliderCtrlWithEcho::~SliderCtrlWithEcho(){}


BEGIN_MESSAGE_MAP(SliderCtrlWithEcho, CSliderCtrl)
  ON_WM_HSCROLL_REFLECT()
  ON_WM_VSCROLL_REFLECT()
  ON_WM_SHOWWINDOW()
END_MESSAGE_MAP()

/// Handler for WM_HSCROLL event. Calls class member function to convert slider
///		value and updates new value to UI area
void UWBGL_MFC::SliderCtrlWithEcho::HScroll(UINT nSBCode, UINT nPos)
{
  float userValue = MFCToUserPos(GetPos());
  UpdateSliderEcho();
}

/// Handler for WM_VSCROLL event. Calls class member function to convert slider
///		value and updates new value to UI area
void UWBGL_MFC::SliderCtrlWithEcho::VScroll(UINT nSBCode, UINT nPos)
{
  float userValue = MFCToUserPos(GetPos());
  UpdateSliderEcho();
}

/// Handler for WM_SHOWWINDOW event. 
/// Used to Show or Hide window element
void UWBGL_MFC::SliderCtrlWithEcho::OnShowWindow(BOOL bShow, UINT nStatus)
{
  CSliderCtrl::OnShowWindow(bShow, nStatus);
  if(IsWindow(m_MessageWnd.m_hWnd))
  {
    if(bShow) m_MessageWnd.ShowWindow(SW_SHOW);
    else m_MessageWnd.ShowWindow(SW_HIDE);
  }
}

/// Initialize the SliderCtrlWithEcho object, determine horizontal or vertical orientation,
///		create and associate the connected text box.  If init argument falls outside of slider 
///		range min < init < max, text output for slider value defaults to 0.00
void SliderCtrlWithEcho::Initialize(float min, float max, float init)
{
  /// determine whether the specified window handle identifies an existing window
  if(!::IsWindow(m_MessageWnd.m_hWnd))
  {
    ///use the control rectangle to determine if this is vertical or horizontally oriented
    ////TODO: Figure out how to check for TBS_HORZ and TBS_VERT
    CRect controlRect;
    GetWindowRect(&controlRect);
    m_bIsVert = controlRect.Width()<controlRect.Height();

    CRect msgRect(controlRect);

    /// set dimensions of text and control rectangles
    if(!m_bIsVert)
    {
      msgRect.left = msgRect.right-50;
      controlRect.right = controlRect.right-50;
    }
    else
    {
      msgRect.left = controlRect.left+controlRect.Width()/2-50/2;
      msgRect.right = msgRect.left+50;
      msgRect.top = controlRect.top;
      msgRect.bottom = msgRect.top+20;

      controlRect.top = controlRect.top+20;
    }

    /// replace the screen coordinates given in CRect object with client coordinates. New coords 
    /// are in hardware coordinates, relative to the upper-left corner of the CWnd client area.
    GetParent()->ScreenToClient(&msgRect);
    GetParent()->ScreenToClient(&controlRect);

    /// create the static control: 
    /// text on control, bitwise declaration of control styles, position/size, parent window
    m_MessageWnd.Create(_T("XXXX"),SS_CENTERIMAGE|WS_CHILD|WS_VISIBLE|SS_SUNKEN,msgRect,GetParent());
    MoveWindow(&controlRect);
  
    CFont* font = GetParent()->GetFont();
    m_MessageWnd.SetFont(font);
  }

  /// set local data members and slider attributes
  fMinRange = min;
  fTotalRange = max - min;
  SetRange(0, kMaxSliderRange, TRUE);
  SetSliderValue(init);
  UpdateSliderEcho();
}

/// Set the value of the slider and the slider position
bool SliderCtrlWithEcho::SetSliderValue(float userValue)
{
  if(userValue<fMinRange || userValue>(fMinRange+fTotalRange)) return false;
  int controlPosition = UserToMFCPos(userValue);
  SetPos(controlPosition);
  UpdateSliderEcho();
  return true;
}

/// Retrieve the current value (position) of the slider
float SliderCtrlWithEcho::GetSliderValue()
{
  if(m_bIsVert) return MFCToUserPos(kMaxSliderRange-GetPos());
  return MFCToUserPos(GetPos());
}

/// External interface to MFC-oriented slider values, converts user-specified 
///		value to internal representation along 100K range
int SliderCtrlWithEcho::UserToMFCPos(float userValue)
{
  int mfcPos = int(((userValue - fMinRange) * kMaxSliderRange / fTotalRange)+0.5f);
  if(m_bIsVert) mfcPos = kMaxSliderRange-mfcPos;
  return mfcPos;
}

/// Internal support for MFC 100K floating point slider range, converts to
///		user-specified value range
float SliderCtrlWithEcho::MFCToUserPos(int mfcValue)
{
  return (fMinRange + ( (float(mfcValue) / float(kMaxSliderRange)) * fTotalRange));
}

/// Set format for text box and update static UI control
void SliderCtrlWithEcho::UpdateSliderEcho() {
  if(IsWindow(m_MessageWnd.m_hWnd))
  {
    CString msg;
    msg.Format(" %.2f",GetSliderValue());
    m_MessageWnd.SetWindowText(msg);
  }
}
