// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////

/// uwbgl_MFCUIWindow.h: header file for a CWnd-derived window
///
#pragma once
#include "../UWB_GraphicsAPI/uwbgl_ViewportBase.h"

namespace UWBGL_MFC
{
  /// An MFC derived CWnd (window) that supports containment of a Viewport. Default implementation
  /// allows for a single viewport that covers the entire window area. Mouse, draw, and resize
  /// events are passed down to contained viewport.
  class UIWindow : public CWnd
  {
  public:
    /// Construct a UIWindow. Note that this does not actually create a window that you see on
    /// the screen. Window creation is done by either calling the Create(...) or
    /// ReplaceDialogControl(...) functions.
    /// \param timerBased If true, the window does not repaint on normal paint messages. All repaint
    ///                  functionality is driven through an external timer function. This is the
    ///                  typical approach for CSS450/410 projects.
    UIWindow(bool timerBased = true);

    /// The destructor does NOT destroy it's viewport. It is up to the project to destroy the actual
    /// viewport object.
    virtual ~UIWindow();

    /// Create this window and "glue" it to a location on a dialog defined by a placeholder control
    /// with the resource editor. This is a convenience function that provides a method to define
    /// UIWindows on a dialog.
    /// \param hDialogHandle Handle to the dialog (m_hWnd class member) that the control replacement
    ///                      is occuring on
    /// \param id Control id for the placeholder control that you define in the resource editor
    /// \return true on successful replacement
    bool ReplaceDialogControl(HWND hDialogHandle, int id);

    /// Set a viewport that this window will contain. The window will call the viewport's
    /// CreateGraphicsDevice.
    /// \return true on success
    bool AttachViewport(IViewport* pViewport);

    /// Call this function inside of your timer routine to draw the viewport.
    void DrawViewport();
  protected:
    DECLARE_MESSAGE_MAP()

    // windows event handler functions; most functions will end up calling a
    // virtual function on the Viewport

    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    afx_msg void OnPaint();
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
    afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
    afx_msg void OnRButtonUp(UINT nFlags, CPoint point);

  protected:

    /// Pointer to the IViewport object
    IViewport* m_pViewport;

    /// Specifies whether window repaints on normal repaint messages
    bool m_bTimerBased;
  };
};
